// Title:   DefiningSurfaces.h
// Created: Thu Sep 25 15:16:22 2003
// Authors: Yong Joo Kil (modified by)
// Authors: Simon Heinzle, Tim Weyrich
// Copyright (c) 2001, 2002, 2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#ifndef __DEFININGSURFACES_H__
#define __DEFININGSURFACES_H__

#include "Action.h"
#include "Selector.h"
#include "PlaneFit.h"
#include "DefiningSurfaceTypes.h"

class DefiningSurfaces :public NormalAction {
	// plane projection
	PlaneFit _planefit;
	Vector3D _plane_projected_position;
	// polynomial projection
	Vector3D _poly_projected_position;

	// general parameters
	unsigned int _neighbourhood_size;
	float _alpha;
	int _degree;
	bool _useplane;
	bool _projectSelectionOnly;
	SurfaceParams _sp;

	void planeProjector(Vector3D source_position, int max_iterations, float min_error);

public:
	DefiningSurfaces();
	void setParameters(int nhood_size, float gauss_factor, float alpha, bool use_plane, int degree, 
			int extremalSurfaceMethod, 
			bool useRadiusAsWeight, float radiusScale,
			float mahalanobisScale,
			int	brentMaxIter, float brentTol, float brentSearchRange, 
			bool projectSelectionOnly);

	void execute();
	UndoAction* executeUndo();
};

#endif

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
