// Title:   MemChecker.cpp
// Created: Thu Sep 25 15:16:34 2003
// Authors: Simon Heinzle, Tim Weyrich
//
// Copyright (c) 2001, 2002, 2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#ifdef _WIN32
	// turn off std::map warnings
	//# pragma warning(push)
	# pragma warning(disable:4786) // identifier truncated to 255 char
	# pragma warning(disable:4284) // return type ... (is not an udt)
#endif

#include "MemChecker.h"
#include <stdio.h>

MemChecker *memcheck = 0;

MemChecker* MemChecker::getInstance() {
	if (memcheck == 0)
		memcheck = new MemChecker;

	return memcheck;
}


void MemChecker::AddTrack(unsigned long addr, unsigned long size, const char *filename, unsigned long line_num) {
	sAllocInfo *info;

	info = new sAllocInfo;
	info->address = addr;
	info->size = size;
	strncpy(info->file, filename, 1023);
	info->line = line_num;

	_list.push_back(info);
}


void MemChecker::RemoveTrack(unsigned long addr) {
	std::list<sAllocInfo*>::iterator it;
	
	for (it = _list.begin(); it != _list.end(); it++) {
		if ((*it)->address == addr) {
			_list.remove(*it);
			break;
		}
	}
}

void MemChecker::DumpUnfreed() {
	std::list<sAllocInfo*>::iterator it;
	unsigned long totalSize = 0;
	char buf[2048];

	for(it = _list.begin(); it != _list.end(); it++) {
		sprintf(buf, "%-50s:\t\tLINE %d,\t\tADDRESS %d\t%d unfreed",
		(*it)->file, (*it)->line, (*it)->address, (*it)->size);
		qDebug(buf);
		totalSize += (*it)->size;
	}
	sprintf(buf, "-----------------------------------------------------------");
	qDebug(buf);
	sprintf(buf, "Total Unfreed: %d bytes", totalSize);
	qDebug(buf);
};

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
