// Title:   Metric.cpp
// Created: Thu Sep 25 15:16:31 2003
// Authors: Simon Heinzle, Tim Weyrich
//
// Copyright (c) 2001, 2002, 2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#include "Metric.h"

#include <qgl.h>

#include "NewOverload.h"

Metric::Metric(float one_unit) {
	_one_unit = one_unit;
	_one_unit_inv = 1.f / one_unit;
}

float Metric::getMetric(float length_) {
	return length_*_one_unit;
}

float Metric::getInverseMetric(float metric_) {
	return metric_*_one_unit_inv;
}

void Metric::showMetricGL(Vector3D center, Vector3D length_) {
	float unit_length;
	float display_length;
	float current_length;
	Vector3D e1, e2;

	float green[4] = { 0.f, 1.f, 0.f, 1.f}, red[4] = { 1.f, 0.f, 0.f, 1.f};
	bool red_on;

	unit_length = getMetric(1);
	
	display_length = length_[0];
	current_length = 0;
	red_on = true;
	
	glPushMatrix();
	glTranslatef(center[0], center[1], center[2]);
	while (current_length < display_length) {
		if (red_on) 
			glMaterialfv(GL_FRONT,GL_DIFFUSE, red);
		else
			glMaterialfv(GL_FRONT,GL_DIFFUSE, green);
		// x axis;
		e1 = Vector3D(current_length,             -THICKNESS,  THICKNESS);
		e2 = Vector3D(current_length+unit_length,  THICKNESS, -THICKNESS);
		showCubeGL(e1,e2);
			
		current_length += unit_length;
		red_on = !red_on;
	}

	display_length = length_[1];
	current_length = 0;
	red_on = true;
	
	while (current_length < display_length) {
		if (red_on) 
			glMaterialfv(GL_FRONT,GL_DIFFUSE, red);
		else
			glMaterialfv(GL_FRONT,GL_DIFFUSE, green);
		// y axis;
		e1 = Vector3D(-THICKNESS, current_length,             THICKNESS);
		e2 = Vector3D( THICKNESS, current_length+unit_length, -THICKNESS);
		showCubeGL(e1,e2);
		
		current_length += unit_length;
		red_on = !red_on;
	}

	display_length = length_[2];
	current_length = 0;
	red_on = true;
	
	while (current_length < display_length) {
		if (red_on) 
			glMaterialfv(GL_FRONT,GL_DIFFUSE, red);
		else
			glMaterialfv(GL_FRONT,GL_DIFFUSE, green);
		// z axis;
		e1 = Vector3D(-THICKNESS, -THICKNESS, current_length);
		e2 = Vector3D( THICKNESS,  THICKNESS, current_length+unit_length);
		showCubeGL(e1,e2);
		
		current_length += unit_length;
		red_on = !red_on;
	}
	glPopMatrix();
}	

void Metric::showCubeGL(Vector3D e1, Vector3D e2) {
	glBegin(GL_QUADS); 
		// Front Face
		glTexCoord2f(0.0f, 0.0f); glVertex3f( e1[0],  e1[1],  e1[2]);	// Bottom Left Of The Texture and Quad
		glTexCoord2f(1.0f, 0.0f); glVertex3f( e2[0],  e1[1],  e1[2]);	// Bottom Right Of The Texture and Quad
		glTexCoord2f(1.0f, 1.0f); glVertex3f( e2[0],  e2[1],  e1[2]);	// Top Right Of The Texture and Quad
		glTexCoord2f(0.0f, 1.0f); glVertex3f( e1[0],  e2[1],  e1[2]);	// Top Left Of The Texture and Quad
		// Back Face
		glTexCoord2f(1.0f, 0.0f); glVertex3f( e1[0],  e1[1],  e2[2]);	// Bottom Right Of The Texture and Quad
		glTexCoord2f(1.0f, 1.0f); glVertex3f( e1[0],  e2[1],  e2[2]);	// Top Right Of The Texture and Quad
		glTexCoord2f(0.0f, 1.0f); glVertex3f( e2[0],  e2[1],  e2[2]);	// Top Left Of The Texture and Quad
		glTexCoord2f(0.0f, 0.0f); glVertex3f( e2[0],  e1[1],  e2[2]);	// Bottom Left Of The Texture and Quad
		// Top Face
		glTexCoord2f(0.0f, 1.0f); glVertex3f( e1[0],  e2[1],  e2[2]);	// Top Left Of The Texture and Quad
		glTexCoord2f(0.0f, 0.0f); glVertex3f( e1[0],  e2[1],  e1[2]);	// Bottom Left Of The Texture and Quad
		glTexCoord2f(1.0f, 0.0f); glVertex3f( e2[0],  e2[1],  e1[2]);	// Bottom Right Of The Texture and Quad
		glTexCoord2f(1.0f, 1.0f); glVertex3f( e2[0],  e2[1],  e2[2]);	// Top Right Of The Texture and Quad
		// Bottom Face
		glTexCoord2f(1.0f, 1.0f); glVertex3f( e1[0],  e1[1],  e2[2]);	// Top Right Of The Texture and Quad
		glTexCoord2f(0.0f, 1.0f); glVertex3f( e2[0],  e1[1],  e2[2]);	// Top Left Of The Texture and Quad
		glTexCoord2f(0.0f, 0.0f); glVertex3f( e2[0],  e1[1],  e1[2]);	// Bottom Left Of The Texture and Quad
		glTexCoord2f(1.0f, 0.0f); glVertex3f( e1[0],  e1[1],  e1[2]);	// Bottom Right Of The Texture and Quad
		// Right face
		glTexCoord2f(1.0f, 0.0f); glVertex3f( e2[0],  e1[1],  e2[2]);	// Bottom Right Of The Texture and Quad
		glTexCoord2f(1.0f, 1.0f); glVertex3f( e2[0],  e2[1],  e2[2]);	// Top Right Of The Texture and Quad
		glTexCoord2f(0.0f, 1.0f); glVertex3f( e2[0],  e2[1],  e1[2]);	// Top Left Of The Texture and Quad
		glTexCoord2f(0.0f, 0.0f); glVertex3f( e2[0],  e1[1],  e1[2]);	// Bottom Left Of The Texture and Quad
		// Left Face
		glTexCoord2f(0.0f, 0.0f); glVertex3f( e1[0],  e1[1],  e2[2]);	// Bottom Left Of The Texture and Quad
		glTexCoord2f(1.0f, 0.0f); glVertex3f( e1[0],  e1[1],  e1[2]);	// Bottom Right Of The Texture and Quad
		glTexCoord2f(1.0f, 1.0f); glVertex3f( e1[0],  e2[1],  e1[2]);	// Top Right Of The Texture and Quad
		glTexCoord2f(0.0f, 1.0f); glVertex3f( e1[0],  e2[1],  e2[2]);	// Top Left Of The Texture and Quad
	glEnd();
}

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
