// Title:   PointNormalSurfacePlaneFit.cpp
// Created: Thu Sep 25 15:15:54 2003
// Authors: Yong Joo Kil (modified by)
// Authors: Simon Heinzle, Tim Weyrich
//
// Copyright (c) 2001, 2002, 2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#ifdef _WIN32
	// turn off std::map warnings
	# pragma warning(push)
	# pragma warning(disable:4786) 
#endif


#include "PointNormalSurfacePlaneFit.h"
#include "PointNormalEnergy.h"

//#include <qgl.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_eigen.h>

#include "Selector.h"
#include "DefiningSurfaceTypes.h"

#include "NewOverload.h"

//project x to plane(q,n)
inline Vector3D project2plane(const Vector3D & q, const Vector3D & n, const Vector3D & x)
{
	float f = Vector3D::dotProduct((x - q),n);
	Vector3D projx = x - f*n;
	return projx;
}

PointNormalSurfacePlaneFit::PointNormalSurfacePlaneFit() {
	_normal = Vector3D(0.,0.,0.);
	_isvalid = false;
}

void PointNormalSurfacePlaneFit::doPlaneFitByKdTreeEnergyRealParticle(const Vector3D &source, KdTreeExtra * kdtree, const SurfaceParams & sp, EnergyFunc ef, OptDirectionFunc nf) {
	float t;
	
	if (sp.gaussFactor== 0) {
		failureHandler();
		return;
	}

	PointNormalEnergy energy(kdtree, sp, ef, nf);

	_normal = energy.find_best_n(source);
	t = energy.find_best_t(source, _normal);
	_center = source + t*_normal;

	_d = -1.*(_normal[0]*_center[0]+_normal[1]*_center[1]+_normal[2]*_center[2]);
	_isvalid = true;
}

void PointNormalSurfacePlaneFit::getEigenValues(float eigenvals[3]) const {
	eigenvals[0] = _eigenvalues[0];
	eigenvals[1] = _eigenvalues[1];
	eigenvals[2] = _eigenvalues[2];
}

Vector3D PointNormalSurfacePlaneFit::getNormal() const {
	return _normal;
}

Vector3D PointNormalSurfacePlaneFit::getCenter() const {
	return _center;
}

float PointNormalSurfacePlaneFit::getD() const  {
	return _d;
}

void PointNormalSurfacePlaneFit::reset() {
	_normal = Vector3D(0.,0.,0.);
}

float PointNormalSurfacePlaneFit::getDistance(Vector3D source) const {
	return Vector3D::dotProduct((source - _center),_normal);
}

bool PointNormalSurfacePlaneFit::isValid() const {
	return _isvalid;
}

void PointNormalSurfacePlaneFit::failureHandler() {
	_isvalid = false;
	_center = _normal = Vector3D(0,0,0);
	_d = 0;
	_eigenvalues[0] = _eigenvalues[1] = _eigenvalues[2] = 0;
}


// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:

