// Title:   PointNormalSurfacePlaneFit.h
// Created: Thu Sep 25 15:15:52 2003
// Authors: Yong Joo Kil (modified by)
// Authors: Simon Heinzle, Tim Weyrich
//
// Copyright (c) 2001, 2002, 2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#ifndef __POINTNORMAL_SURFACE_PLANEFIT_H
#define __POINTNORMAL_SURFACE_PLANEFIT_H

#include <vector>
#include "../../../../DataTypes/src/Vector3D.h"
#include "../../../../Core/DataStructures/src/QueryGrid.h"
#include "../../../../Core/DataStructures/src/NeighbourHood.h"

#include "DefiningSurfaceTypes.h"
#include "KdTreeExtra.h"
#if QT_VERSION >= 0x030000
# include <qptrvector.h>
# ifndef QVector
#  define QVector QPtrVector
# endif
# ifndef QVectorIterator
#  define QVectorIterator QPtrVectorIterator
# endif
#else
# include <qvector.h>
#endif

/**
 * Plane fit class
 *	Calculates a plane of form:
 *		n[0]*x + n[1]*y + n[2]*z + d = 0;
 *
 * @author Simon Heinzle
 * @version 1.0
 */

typedef unsigned int* uIntPtr;
typedef unsigned int  uInt;

class PointNormalSurfacePlaneFit {
	Vector3D _normal;
	Vector3D _center;
	float _d;
	float _eigenvalues[3];
	bool _isvalid;
	
	/**
	 * internal failure handler
	 */
	void failureHandler();
public:
	/**
	 * Constructor
	 */
	PointNormalSurfacePlaneFit();

	void doPlaneFitByKdTreeEnergyRealParticle(const Vector3D &source, KdTreeExtra * kdtree, const SurfaceParams & sp, EnergyFunc ef, OptDirectionFunc nf);
	
	/**
	 * Returns the eigenvalues of the PCA in <code>float eigenvals[3]</code>
	 */
	void getEigenValues(float eigenvals[3]) const;
	/**
	 * @return the normal of the plane fit
	 */
	Vector3D getNormal() const;
	/**
	 * @return the center of the plane fit
	 */
	Vector3D getCenter() const;
	/**
	 * @return the d parameter of the plane
	 */
	float getD() const;
	/**
	 * Resets the plane fit
	 */
	void reset();
	/**
	 * @return the distance from <code>Vector3D source</code> to the plane
	 */
	float getDistance(Vector3D source) const;
	/**
	 * @return <code>bool</code> to indicate if planefit is valid.
	 */
	bool isValid() const;
};

#endif

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
