// Title:   vectors.h
// Created: Thu Sep 25 15:14:29 2003
// Authors: Simon Heinzle, Tim Weyrich
//
// Copyright (c) 2001, 2002, 2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//


/*
**	Vector - Template
**
**	Problem: sqrt in Length is not at all generic
*/

#ifndef __VECTORS_H
#define __VECTORS_H

#include <iostream>
using namespace std;


template<typename T>
class cVector3D {
	T _coordinates[3];
public:
	cVector3D(void) { }
	cVector3D (T x, T y, T z) { _coordinates[0]=x; _coordinates[1]=y; _coordinates[2]=z;}
	
	void Normalize(void);
	T Length(void);

	T* GetPtr(void);
	void SetPtr(T *xyz) { for (int i=0;i<3;i++) _coordinates[i] = xyz[i]; }
	void SetValue(T xx, T yy, T zz) { _coordinates[0]=xx; _coordinates[1]=yy; _coordinates[2]=zz; }
		
	T& operator[](int index) { 
		#ifdef DEBUG_BOUNDARY_CHECK
			if(index<0||index>2)
				cout << "Vector index out of bounds, tried to access item nr. " << index << "!" << endl;
		#endif
		
		return _coordinates[index]; 
	}
	
	cVector3D operator+(cVector3D a);
	cVector3D operator+=(cVector3D a);
	cVector3D operator-(cVector3D a);
	cVector3D operator-=(cVector3D a);
	T operator*(cVector3D a);
	cVector3D operator*(T a);
	cVector3D operator*=(T a);
	cVector3D operator/(T a);
	cVector3D operator/=(T a);
	
	cVector3D operator%(cVector3D a) const; // the normal !!!

	// following test operators test all elements to fullfill condition
	bool operator<(cVector3D& a);
	bool operator<=(cVector3D& a);
	bool operator>=(cVector3D& a);
	bool operator>(cVector3D& a);
	bool operator==(cVector3D& a);
	

	// NULLVECTOR
	static NullVector(void) {
		cVector3D<T> return_value(0,0,0);
		return return_value;
	}

	void DebugPrint() { cout << _coordinates[0] << " ~ " << _coordinates[1] << " ~ " << _coordinates[2] << endl; }
};

template<typename T>
T* cVector3D<T>::GetPtr(void)
{
  return _coordinates;
}

template<typename T>
T cVector3D<T>::Length(void)
{
  return sqrt(_coordinates[0]*_coordinates[0]+_coordinates[1]*_coordinates[1]+_coordinates[2]*_coordinates[2]);
}

template<typename T>
void cVector3D<T>::Normalize(void)
{
  T len=Length();
  
  // next one would be nice, but not that possible...
  // if (T == 0) return;
  
  _coordinates[0]/=len;
  _coordinates[1]/=len;
  _coordinates[2]/=len;
}

/*
**	operator* overload for T-Classes
*/
/*
template<typename T>
cVector3D<T> operator*(const T a,Vector3D<T> b)
{
  cVector3D<T> tmp(b[0]*a, b[1]*a, b[2]*a);
  return tmp;
}
*/

template<typename T>
cVector3D<T> cVector3D<T>::operator+=(cVector3D<T> a)
{
  _coordinates[0]+=a[0];
  _coordinates[1]+=a[1];
  _coordinates[2]+=a[2];
  return *this;
}

template<typename T>
cVector3D<T> cVector3D<T>::operator+(cVector3D<T> a)
{
  cVector3D<T> tmp;
  tmp[0]=_coordinates[0]+a[0];
  tmp[1]=_coordinates[1]+a[1];
  tmp[2]=_coordinates[2]+a[2];
  return tmp;
}

template<typename T>
cVector3D<T> cVector3D<T>::operator-=(cVector3D<T> a)
{
  _coordinates[0]-=a[0];
  _coordinates[1]-=a[1];
  _coordinates[2]-=a[2];
  return *this;
}

template<typename T>
cVector3D<T> cVector3D<T>::operator-(cVector3D<T> a)
{
  cVector3D<T> tmp;
  tmp[0]=_coordinates[0]-a[0];
  tmp[1]=_coordinates[1]-a[1];
  tmp[2]=_coordinates[2]-a[2];
  return tmp;
}

template<typename T>
T cVector3D<T>::operator*(cVector3D<T> a)
{
  return(_coordinates[0]*a[0] + _coordinates[1]*a[1] + _coordinates[2]*a[2]);
}

template<typename T>
cVector3D<T> cVector3D<T>::operator*(T a)
{
  cVector3D<T> tmp;

  tmp[0]=_coordinates[0]*a;
  tmp[1]=_coordinates[1]*a;
  tmp[2]=_coordinates[2]*a;
  
  return tmp;
}

template<typename T>
cVector3D<T> cVector3D<T>::operator*=(T a)
{
  _coordinates[0]*=a;
  _coordinates[1]*=a;
  _coordinates[2]*=a;
  
  return *this;
}

template<typename T>
cVector3D<T> cVector3D<T>::operator/(T a) {
	  cVector3D<T> tmp;

  tmp[0]=_coordinates[0]/a;
  tmp[1]=_coordinates[1]/a;
  tmp[2]=_coordinates[2]/a;
  
  return tmp;
}

template<typename T>
cVector3D<T> cVector3D<T>::operator/=(T a) {
	_coordinates[0]*=a;
  _coordinates[1]*=a;
  _coordinates[2]*=a;
  
  return *this;
}

template<typename T>
cVector3D<T> cVector3D<T>::operator%(cVector3D<T> a) const
{
  cVector3D<T> tmp;

	tmp[0] = _coordinates[1]*a[2] - _coordinates[2]*a[1];
	tmp[1] = _coordinates[2]*a[0] - _coordinates[0]*a[2];
	tmp[2] = _coordinates[0]*a[1] - _coordinates[1]*a[0];
  
  return tmp;
}

template<typename T>
bool cVector3D<T>::operator<(cVector3D<T>& a) {
	return ( this->_coordinates[0]<a._coordinates[0]
				&& this->_coordinates[1]<a._coordinates[1]
				&& this->_coordinates[2]<a._coordinates[2]);
}

template<typename T>
bool cVector3D<T>::operator<=(cVector3D<T>& a){
	return ( this->_coordinates[0]<=a._coordinates[0]
				&& this->_coordinates[1]<=a._coordinates[1]
				&& this->_coordinates[2]<=a._coordinates[2]);
}

template<typename T>
bool cVector3D<T>::operator>=(cVector3D<T>& a){
	return ( this->_coordinates[0]>=a._coordinates[0]
				&& this->_coordinates[1]>=a._coordinates[1]
				&& this->_coordinates[2]>=a._coordinates[2]);
}

template<typename T>
bool cVector3D<T>::operator>(cVector3D<T>& a){
	return ( this->_coordinates[0]>a._coordinates[0]
				&& this->_coordinates[1]>a._coordinates[1]
				&& this->_coordinates[2]>a._coordinates[2]);
}

template<typename T>
bool cVector3D<T>::operator==(cVector3D<T>& a){
	return ( this->_coordinates[0]==a._coordinates[0]
				&& this->_coordinates[1]==a._coordinates[1]
				&& this->_coordinates[2]==a._coordinates[2]);
}

#endif

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
